/*
 * xreplot - Function plotting utility
 * Copyright(c) David Drysdale 1995-1996
 * Freely distributable provided this copyright notice remains intact
 *
 * Send bug reports to dmd@lurklurk.org
 */



#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "xutil.h"
#include "xreplot.h"

/*****************************************************************************/
/* Postscript Plotting routines                                              */
/*****************************************************************************/

void ps_real_func_sl(float voffset, float y, float vstep)
{
  fprintf(outfile, "0 %f moveto\n", (voffset+y));
}
void ps_real_func_lp(float voffset, float x, float y, float vstep)
{
  fprintf(outfile, "%f %f lineto\n", x, (voffset+y));
}
void ps_real_func_el()
{
  fprintf(outfile, "stroke\n");
}

void ps_real_gray_sl(float voffset, float y, float vstep)
{
  fprintf(outfile, "/voff %f def\n0 ", voffset);
}
void ps_real_gray_lp(float voffset, float x, float y, float vstep)
{
  fprintf(outfile, "%f %f gd\n", x, y);
}
void ps_real_gray_el()
{
  fprintf(outfile, "pop\n");
}

void ps_complex_gray_sl(float voffset, float y0, float theta, float x1, float y1, float vstep)
{
  fprintf(outfile, "/voff %f def\n0 ", voffset);
  fprintf(outfile, "%f %f %f cgd\n", x1, y1, theta);
}
void ps_complex_gray_lp(float voffset, float theta, float x, float y, float vstep)
{
  fprintf(outfile, "%f %f %f cgd\n", x, y, theta);
}
void ps_complex_gray_el()
{
  fprintf(outfile, "pop 0 1 0 sethsbcolor\n");
}

void ps_complex_func_sl(float voffset, float y0, float theta, float x1, float y1, float vstep)
{
  fprintf(outfile, "0 %f %f %f %f dd\n", (voffset+y0), theta, x1, (voffset+y1));
}
void ps_complex_func_lp(float voffset, float theta, float x, float y, float vstep)
{
  fprintf(outfile, "%.4f %.4f %.4f dd\n", theta, x, (voffset+y));
}
void ps_complex_func_el()
{
  /* remove extra x,y and set color back to black */
  fprintf(outfile, "pop pop 0 1 0 sethsbcolor\n");
}
void ps_printtick(float temp, float lasttickprinttime, float fontsize,
                  float tick, float doubled, int grayscale)
{
  if (grayscale) fprintf(outfile, "0 setgray ");
  fprintf(outfile, "0 %lf moveto\n", temp);
  fprintf(outfile, "(%.2f  ) dup stringwidth pop neg ", lasttickprinttime);
  fprintf(outfile, " %f rmoveto show\n",(-fontsize/3.0));
  fprintf(outfile, "%lf %lf moveto 0 %lf lineto stroke\n", -tick/doubled, temp,temp);
}

/*****************************************************************************/
/* Postscript prolog output function                                         */
/*****************************************************************************/
void ps_prolog(char *options, int norows, float voffmax, float vstep,
               float pagewidth, float *deltax, float scaling, float originx,
               float originy, float tick, float fontsize, float maxnorm,
               float doubled, int complexin, int grayscale, int reverse)
{
  if (outfile==NULL) outfile = stdout;

  *deltax = pagewidth / ((double) norows - 1.0);
  fprintf(outfile, "%%!\n%%%%Title: cgl plot\n%%%%Creator: xreplot\n");
  /* fprintf(outfile, "%%%%CreationDate: %s\n", date);
  fprintf(outfile, "%%%%For:\n");
  fprintf(outfile, "%%%%Pages: 1\n"); */
  fprintf(outfile, "%%%%Options: ");
  if (complexin)
    fprintf(outfile, "-c ");
  if (grayscale==1)
    fprintf(outfile, "-g ");
  else if (grayscale==-1)
    fprintf(outfile, "-i ");
  fprintf(outfile, "%s\n", options);

  fprintf(outfile, "%%%%BoundingBox: %f %f %f %f\n",
         (originx-fontsize*4.0)/doubled,
         originy -(fontsize/3.0),
         originx+pagewidth+fontsize*4.0+18,
         originy+voffmax+scaling*10.0);
  fprintf(outfile, "%%%%EndComments\n");
  if (grayscale) {
    if (complexin) {
      fprintf(outfile, "/cgd { %% xleft xright graylevel phase gd => xright\n");
      fprintf(outfile, "/theta exch def /gray exch def /xr exch def /xl exch def\n");
      fprintf(outfile, "newpath xl voff moveto xr xl sub 0 rlineto 0 %f rlineto\n", vstep);
      if (grayscale==-1) {
        fprintf(outfile, "xl xr sub 0 rlineto closepath theta 1 gray sub 1 sethsbcolor fill xr } def\n");
      } else {
        fprintf(outfile, "xl xr sub 0 rlineto closepath theta gray 1 sethsbcolor fill xr } def\n");
      }
    } else {
      fprintf(outfile, "/gd { %% xleft xright graylevel gd => xright\n");
      fprintf(outfile, "/gray exch def /xr exch def /xl exch def\n");
      fprintf(outfile, "newpath xl voff moveto xr xl sub 0 rlineto 0 %f rlineto\n", vstep);
      if (grayscale==1) {
        fprintf(outfile, "xl xr sub 0 rlineto closepath 1 gray sub setgray fill xr } def\n");
      } else {
        fprintf(outfile, "xl xr sub 0 rlineto closepath gray setgray fill xr } def\n");
      }
    }
  }
  if (complexin) {
    fprintf(outfile, "/dd {/y exch def /x exch def 1 1 sethsbcolor \n");
    fprintf(outfile, "     moveto x y lineto stroke x y} def\n");
    fprintf(outfile, "/theta {360 mul n div} def /colii {n div} def\n");
    fprintf(outfile, "/colourcircle { gsave translate /r exch def /n exch def /ii 1 def\n");
    fprintf(outfile, "  4 setlinewidth\n");
    fprintf(outfile, "  /olddd {moveto 1 1 sethsbcolor lineto stroke} def\n");
    fprintf(outfile, "  1 1 n { pop r ii 1 sub theta sin mul\n");
    fprintf(outfile, "          r ii 1 sub theta cos mul neg\n");
    fprintf(outfile, "          ii colii\n");
    fprintf(outfile, "          r ii theta sin mul\n");
    fprintf(outfile, "          r ii theta cos mul neg\n");
    fprintf(outfile, "          olddd ii 1 add /ii exch def} for\n");
    fprintf(outfile, "  0 1 0 sethsbcolor\n");
    fprintf(outfile, "  1 setlinewidth\n");
    fprintf(outfile, "  0 0 moveto\n");
    fprintf(outfile, "  0 1.5 r mul lineto stroke\n");
    fprintf(outfile, "  0 1.5 r mul moveto ( 0) show grestore} def\n");
  }
  fprintf(outfile, "%lf %lf translate\n", originx/doubled, originy);
  fprintf(outfile, "/Times-Roman findfont %f scalefont setfont\n", fontsize/doubled);

  /* Do the norm scale at the right hand side */
  if (fontsize != 0.0) {
    if (!grayscale) {
      fprintf(outfile, "%lf 0 moveto ", (pagewidth+12)/doubled);
      fprintf(outfile, "%lf %lf lineto ", (pagewidth+12)/doubled, (scaling*maxnorm) );
      fprintf(outfile, "%lf 0 moveto ", (pagewidth+12)/doubled);
      fprintf(outfile, "%lf 0 rlineto ", tick/doubled);
      fprintf(outfile, "0 %lf rmoveto ", (-fontsize / 3.0) );
      fprintf(outfile, "(  %.2f) show\n", 0.0);
      fprintf(outfile, "%lf %lf moveto ", (pagewidth+12)/doubled, (scaling*maxnorm) );
      fprintf(outfile, "%lf 0 rlineto ", tick/doubled);
      fprintf(outfile, "0 %lf rmoveto ", (-fontsize / 3.0) );
      fprintf(outfile, "(  %.2f) show\n", maxnorm);
      fprintf(outfile, "%lf %lf moveto ", (pagewidth+12)/doubled, (scaling*maxnorm*.5) );
      fprintf(outfile, "%lf 0 rlineto ", tick/doubled);
      fprintf(outfile, "0 %lf rmoveto ", (-fontsize / 3.0)/doubled );
      fprintf(outfile, "(  %.2f) show\n", (maxnorm*.5));
      fprintf(outfile, "stroke\n");
    } else {
      if (grayscale==1) {
        fprintf(outfile, 
"/grayscl { %% dx2 dy x y => -\n"
"gsave translate /dy exch def /dx exch def\n"
"/h dy 50 div def 5 setlinewidth\n"
"0 1 50 { /ii exch def \n"
"         dx neg ii h mul moveto \n"
"         1 ii 50 div sub setgray\n"
"         2 dx mul 0 rlineto stroke} for\n"
"0 setlinewidth 0 setgray dx neg 0 moveto\n"
"2 dx mul 0 rlineto 0 dy rlineto 2 dx mul neg 0 rlineto 0 dy neg rlineto stroke\n"
"grestore } def\n");
      } else {
        fprintf(outfile, 
"/grayscl { %% dx2 dy x y => -\n"
"gsave translate /dy exch def /dx exch def\n"
"/h dy 50 div def 5 setlinewidth\n"
"0 1 50 { /ii exch def \n"
"         dx neg ii h mul moveto \n"
"         ii 50 div setgray\n"
"         2 dx mul 0 rlineto stroke} for\n"
"0 setlinewidth 0 setgray dx neg 0 moveto\n"
"2 dx mul 0 rlineto 0 dy rlineto 2 dx mul neg 0 rlineto 0 dy neg rlineto stroke\n"
"grestore } def\n");
      }

      fprintf(outfile, "%f %lf %lf 0 grayscl ", tick/doubled, 3.0*maxnorm,
                                      (pagewidth+12)/doubled);

      fprintf(outfile, "%lf %lf moveto ", (pagewidth+12+tick)/doubled,(-fontsize / 3.0));
      fprintf(outfile, "(  %.2f) show\n", 0.0);
      fprintf(outfile, "%lf %lf moveto ", (pagewidth+12+tick)/doubled,
                                (3.0*maxnorm)+(-fontsize / 3.0) );
      fprintf(outfile, "(  %.2f) show\n", 1.0/(scaling*GRAYSCALE));
      fprintf(outfile, "%lf %lf moveto ", (pagewidth+12+tick)/doubled,
             (3.0*maxnorm*.5)+((-fontsize / 3.0)/doubled ) );
      fprintf(outfile, "(  %.2f) show\n", 1.0/(scaling*2.0*GRAYSCALE));
      fprintf(outfile, "stroke\n");
    }
  } /* if fontsize is not zero */

  /* do the phase colour circle */
  if (complexin && (fontsize!=0.0)) {
    if (grayscale) {
      fprintf(outfile, "50 %f %lf %lf colourcircle\n", 20.0/doubled,
             (pagewidth+30.0)/doubled, (maxnorm*6.0) );
    } else {
      fprintf(outfile, "50 %f %lf %lf colourcircle\n", 20.0/doubled,
             (pagewidth+30.0)/doubled, (scaling*maxnorm*3.0) );
    }
  }
  if (reverse==-1) {
    fprintf(outfile, "0 %f translate\n", voffmax);
  }
}

/*****************************************************************************/
/* Output postscript end of page function                                    */
/*****************************************************************************/
void ps_eopage(float voffset, float pagewidth, float vstep,
               float doubled, float originx, float originy,
               float fontsize, int grayscale, int whichpage, int lr)
{
  if (grayscale) fprintf(outfile, "0 setgray ");
  fprintf(outfile, "0 0 moveto\n");
  fprintf(outfile, "0 %lf lineto stroke\n", (voffset-vstep) );
  if (whichpage == 1)
    fprintf(outfile, "0 0 moveto %lf 0 lineto stroke\n", pagewidth/doubled);
  if (lr == 1) {
    fprintf(outfile, "showpage\n");
    fprintf(outfile, "%lf %lf translate\n", originx/doubled, originy);
  } else {
    /* first move origin back to bottom left of page */
    fprintf(outfile, "%lf %lf translate\n", -originx/doubled, -originy);
    fprintf(outfile, "%lf %lf translate\n",
           originx/doubled+(pagewidth+64+8.0*fontsize)/doubled, originy);
  }
}

/*****************************************************************************/
/* Output postscript epilog function                                         */
/*****************************************************************************/
void ps_epilog(int grayscale, float voffset, int whichpage,
               float pagewidth, float doubled)
{
  if (grayscale) fprintf(outfile, "0 setgray ");
  fprintf(outfile, "0 0 moveto\n");
  fprintf(outfile, "0 %lf lineto stroke\n", voffset );
  if (whichpage == 1)
    fprintf(outfile, "0 0 moveto %lf 0 lineto stroke\n", pagewidth/doubled);
  fprintf(outfile, "showpage\n");

  if (outfile==stdout) outfile=NULL;
}
