/*
 * xreplot - Function plotting utility
 * Copyright(c) David Drysdale 1995-1996
 * Freely distributable provided this copyright notice remains intact
 *
 * Send bug reports to dmd@lurklurk.org
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>


#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Core.h>

#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/AsciiSrc.h>
#include <X11/Xaw/Dialog.h>

#define XREPLOT
#include "xutil.h"
#include "xreplot.h"

#include "xreplot.bmp"
#include "xreploto.bmp"


/*****************************************************************************/
/* Widget drawing routines                                                   */
/*****************************************************************************/
extern Widget pvshell;
extern Widget viewwin;
Widget drawwin;
Colormap cmap = 0;
int lastx, lasty;
GC drawgc;
int screen_num;
XFontStruct *font_info;
int font_loaded = 0;
float fontsize_loaded;
char fontname[256];
float origx;
float origy;
float voffmxx;
float x_gray_voffset;
float x_gray_x;
unsigned long hsb[NUMXCOLS+1];
int hsballoced=0;
unsigned long hsb2[NUMXHUES+1][NUMXBRIGHTS+1];
int hsb2alloced=0;
unsigned long grays[NUMXHUES+1];
int grayalloced=0;
int xgrayscale;
Pixel black_pixel;
Pixel white_pixel;

void x_real_func_sl(float voffset, float y, float vstep)
{
  lastx = 0;
  lasty = (int)(voffset+y);
}
void x_real_func_lp(float voffset, float x, float y, float vstep)
{
  XDrawLine(display, XtWindow(drawwin),
            drawgc,
            (int) origx + lastx, (int) voffmxx - (int) origy - lasty,
            (int) (origx + x), (int) (voffmxx - origy - voffset - y));
  lastx = (int) x;
  lasty = (int) (voffset+y);
}
void x_real_func_el()
{
  XFlush(display);
}

void x_real_gray_sl(float voffset, float y, float vstep)
{
  x_gray_voffset = voffset;
  x_gray_x = 0.0;
}
void x_real_gray_lp(float voffset, float x, float y, float vstep)
{
  float gray;
  gray = (y>1.0) ? 1.0 : y;
  gray = (xgrayscale == 1) ? gray : (1.0-gray);
  XSetForeground(display, drawgc, grays[(int)(gray*(float)NUMXHUES)]);
  if (vstep>0) {
    XFillRectangle(display, XtWindow(drawwin), drawgc,
                 (int)(origx + x_gray_x),
                (int)(voffmxx - origy) - (int)(x_gray_voffset+fabs(vstep)+0.5),
                 (int)(x-x_gray_x)+1, (int)fabs(vstep)+1);
  } else {
    XFillRectangle(display, XtWindow(drawwin), drawgc,
                 (int)(origx + x_gray_x),
                (int)(voffmxx - origy) - (int)(x_gray_voffset+0.5),
                 (int)(x-x_gray_x)+1, (int)fabs(vstep)+1);
  }
  x_gray_x = x;
}
void x_real_gray_el()
{
  XSetForeground(display, drawgc, black_pixel);
  XFlush(display);
}

void x_complex_gray_lp(float voffset, float theta, float x, float y, float vstep)
{
  float gray;
  gray = (y>1.0) ? 1.0 : y;
  gray = (xgrayscale == 1) ? gray : (1.0-gray);
  XSetForeground(display, drawgc,
          hsb2[(int)(theta*(float)NUMXHUES)][(int)(gray*(float)NUMXBRIGHTS)]);
  if (vstep>0) {
    XFillRectangle(display, XtWindow(drawwin), drawgc,
                 (int)(origx + x_gray_x),
                (int)(voffmxx - origy) - (int)(x_gray_voffset+fabs(vstep)+0.5),
                 (int)(x-x_gray_x)+1, (int)fabs(vstep)+1);
  } else {
    XFillRectangle(display, XtWindow(drawwin), drawgc,
                 (int)(origx + x_gray_x),
                (int)(voffmxx - origy) - (int)(x_gray_voffset+0.5),
                 (int)(x-x_gray_x)+1, (int)fabs(vstep)+1);
  }
  x_gray_x = x;
}
void x_complex_gray_sl(float voffset, float y0, float theta, float x1, float y1, float vstep)
{
  x_gray_voffset = voffset;
  x_gray_x = 0.0;
  x_complex_gray_lp(voffset, theta, x1, y1, vstep);
}
void x_complex_gray_el()
{
  XSetForeground(display, drawgc, black_pixel);
}

void x_complex_func_sl(float voffset, float y0, float theta, float x1, float y1, float vstep)
{
  x_real_func_sl(voffset, y0, vstep);
}
void x_complex_func_lp(float voffset, float theta, float x, float y, float vstep)
{
  XSetForeground(display, drawgc, hsb[(int)(theta*(float)NUMXCOLS)]);
  x_real_func_lp(voffset, x, y, vstep);
}
void x_complex_func_el()
{
  /* set color back to black */
  XSetForeground(display, drawgc, black_pixel);
  x_real_func_el();
}
void x_printtick(float temp, float lasttickprinttime, float fontsize,
                  float tick, float doubled, int grayscale)
{
  char tickstring[256];
  int width;

  /* only print ticks if the font is loaded and non-zero size */
  if (font_loaded && (fontsize_loaded != 0.0)) {
    if (grayscale)
      XSetForeground(display, drawgc, black_pixel);
    sprintf(tickstring, "%.2f  ", lasttickprinttime);
    width = XTextWidth(font_info, tickstring, strlen(tickstring));
    XDrawString(display, XtWindow(drawwin), drawgc,
               (int)(origx-width),
               (int)(voffmxx - (temp+origy - fontsize_loaded/3.0)),
               tickstring, strlen(tickstring));
    XDrawLine(display, XtWindow(drawwin), drawgc,
              (int) (origx-tick/doubled) , (int) (voffmxx - origy - temp),
              (int) origx, (int) (voffmxx - origy - temp));
  }
}
void x_complex_prolog(char *options, int norows, float voffmax, float vstep,
               float pagewidth, float *deltax, float scaling, float originx,
               float originy, float tick, float fontsize, float maxnorm,
               float doubled, int complexin, int grayscale, int reverse)
{
  XGCValues values;
  unsigned long valuemask;
  char tickstring[256];
  int ii, jj;
  int nocols;
  XColor black = {0, 0,0,0,               DoRed|DoGreen|DoBlue, 0};
  XColor white = {0, 65535, 65535, 65535, DoRed|DoGreen|DoBlue, 0};
  XColor coldef;
  char colstring[64];
  XVisualInfo visual_info;
  Visual *default_visual;
  int default_depth;
  int vclass = 5;
  int rc;
  int x,y;
  float dx,dy;
  float gray;
  float red, green, blue;
  XSetWindowAttributes winattr;
  XFontStruct *newfont_info;

  *deltax = pagewidth / ((double) norows - 1.0);
  xgrayscale = grayscale;
  origx = originx/doubled;
  origy = originy;
  display = XtDisplay(drawwin);
  screen_num = DefaultScreen(display);
  default_depth = DefaultDepth(display, screen_num);
  default_visual = DefaultVisual(display, screen_num);
  if (cmap == 0) cmap = DefaultColormap(display, screen_num);
  black_pixel = BlackPixel(display, screen_num);
  white_pixel = WhitePixel(display, screen_num);

  winattr.backing_store = WhenMapped;
  XChangeWindowAttributes(display, XtWindow(drawwin), CWBackingStore, &winattr);
  drawgc = XCreateGC(display, XtWindow(drawwin), 0, &values);

  /* load up a font if the size is non zero */
  if (fontsize != 0) {
    sprintf(fontname, "*times-medium-r-*-%d-*", (int) (fontsize*10));
    if ((newfont_info = XLoadQueryFont(display, fontname)) == NULL) {
      fprintf(stderr, "Cannot open font %s\n", fontname);
      if (!font_loaded) {
        exit(1);
      }
    } else {
      if (font_loaded) {
         XFreeFont(display, font_info);
      }
      XSetFont(display, drawgc, newfont_info->fid);
      font_info = newfont_info;
      font_loaded = 1;
      fontsize_loaded = fontsize;
    }
  } else if (font_loaded) {
    XFreeFont(display, font_info);
    font_loaded = 0;
    fontsize_loaded = 0.0;
  }
  XSetForeground(display, drawgc, black_pixel);
  XSetLineAttributes(display, drawgc, 1, LineSolid, CapButt, JoinMiter);

  if (complexin) {
    rc = 0;
    while ((!rc) && (vclass>=0)) {
      rc = XMatchVisualInfo(display, screen_num, default_depth,
                            vclass, &visual_info);
      vclass--;
    }
    ++vclass;
    if (vclass < StaticColor) {
      fprintf(stderr, "No color visual available\n");
      exit(1);
    }
    vclass = default_visual->class;

    coldef.flags = DoRed | DoGreen | DoBlue;
    if (!grayscale) {
      if (grayalloced) {
        XFreeColors(display, cmap, grays, NUMXHUES+1, 0L);
        grayalloced = 0;
      }
      if (hsb2alloced) {
        for (ii=0; ii<=NUMXHUES; ii++) {
          XFreeColors(display, cmap, hsb2[ii], NUMXBRIGHTS+1, 0L);
        }
        hsb2alloced = 0;
      }
      if (!hsballoced) {
        /* load up NUMXCOLS colours of form HSB=(i/NUMXCOLS,1.0,1.0)  */
HSB_AGAIN:
        for (ii=0; ii<=NUMXCOLS; ii++) {
          HSBtoRGB(((float)ii/(float)NUMXCOLS),1.0,1.0,&red,&green,&blue);
          coldef.red   = (short)(65535.0*red);
          coldef.green = (short)(65535.0*green);
          coldef.blue  = (short)(65535.0*blue);
          if (!XAllocColor(display, cmap, &coldef)) {
            if ((vclass == PseudoColor) && (cmap == DefaultColormap(display, screen_num))) {
              /* we have failed to allocate a color in the default colormap */
              /* since this is a pseudo-color visual, allocate our own colormap */
              /* instead */
              fprintf(stderr, "failed to alloc color %d; creating private colormap\n", ii);
              XFreeColors(display, cmap, hsb, ii, 0L);
              cmap = XCreateColormap(display, XtWindow(drawwin), default_visual, AllocNone);
              XAllocColor(display, cmap, &black);
              black_pixel = black.pixel;
              XAllocColor(display, cmap, &white);
              white_pixel = white.pixel;
              XtVaSetValues(drawwin, XtNcolormap, cmap, NULL);
              XtVaSetValues(pvshell, XtNcolormap, cmap, NULL);
              XtVaSetValues(viewwin, XtNcolormap, cmap, NULL);
              goto HSB_AGAIN;
            } else {
              fprintf(stderr, "failed to allocate hsb color %d class %d\n", ii, vclass);
              exit(1);
            }
          }
          hsb[ii] = coldef.pixel;
        }

        hsballoced = 1;
      }
    } else {
      if (grayalloced) {
        XFreeColors(display, cmap, grays, NUMXHUES+1, 0L);
        grayalloced = 0;
      }
      if (hsballoced) {
        XFreeColors(display, cmap, hsb, NUMXCOLS+1, 0L);
        hsballoced = 0;
      }
      if (!hsb2alloced) {
        /* load up NUMXHUES x NUMXBRIGHTS colours of form  */
        /* HSB=(i/NUMXHUES,1.0,j/NUMXBRIGHTS)  */
HSB2_AGAIN:
        for (ii=0; ii<=NUMXHUES; ii++) {
          for (jj=0;  jj<=NUMXBRIGHTS; jj++) {
            HSBtoRGB(((float)ii/(float)NUMXHUES),1.0,
                      (float)jj/(float)NUMXBRIGHTS,&red,&green,&blue);
            coldef.red   = (short)(65535.0*red);
            coldef.green = (short)(65535.0*green);
            coldef.blue  = (short)(65535.0*blue);
            if (!XAllocColor(display, cmap, &coldef)) {
              if ((vclass == PseudoColor) && (cmap == DefaultColormap(display, screen_num))) {
                int kk;
                /* we have failed to allocate a color in the default colormap */
                /* since this is a pseudo-color visual, allocate our own colormap */
                /* instead */
                fprintf(stderr, "failed to alloc color %d, %d; creating private colormap\n", ii, jj);
                for (kk=0; kk<ii; kk++) {
                  XFreeColors(display, cmap, hsb2[kk], NUMXBRIGHTS+1, 0L);
                }
                XFreeColors(display, cmap, hsb2[ii], jj, 0L);
                cmap = XCreateColormap(display, XtWindow(drawwin), default_visual, AllocNone);
                XAllocColor(display, cmap, &black);
                black_pixel = black.pixel;
                XAllocColor(display, cmap, &white);
                white_pixel = white.pixel;
                XtVaSetValues(drawwin, XtNcolormap, cmap, NULL);
                XtVaSetValues(pvshell, XtNcolormap, cmap, NULL);
                XtVaSetValues(viewwin, XtNcolormap, cmap, NULL);
                goto HSB2_AGAIN;
              } else {
                fprintf(stderr, "failed to allocate hsb2 color %d , %d\n", ii ,jj);
                exit(1);
              }
            }
            hsb2[ii][jj] = coldef.pixel;
          }
        }
        hsb2alloced = 1;
      }
    }
    /* Now draw the phase colour circle */
    if (fontsize != 0.0) {
      x=(int)((pagewidth+30.0)/doubled);
      y=(int)(scaling*maxnorm*3.0);
      if (grayscale) y+=(int)(5.0*maxnorm);
      XSetLineAttributes(display, drawgc, 4, LineSolid, CapButt, JoinBevel);
      nocols = (grayscale ? NUMXHUES : NUMXCOLS);
      for (ii = 1; ii<=nocols; ii++) {
        if (grayscale) {
          XSetForeground(display, drawgc, hsb2[ii-1][NUMXBRIGHTS]);
        } else {
          XSetForeground(display, drawgc, hsb[ii-1]);
        }
        XDrawLine(display, XtWindow(drawwin), drawgc,
            x + (int) origx +
               (int)((20.0/doubled) *sin(2.0*PI*(float)ii/(float)nocols)),
            (int) voffmxx - (int) origy - y +
               (int)((20.0/doubled)*cos(2.0*PI*(float)ii/(float)nocols)),
            x + (int) origx +
               (int)((20.0/doubled)*sin(2.0*PI*(float)(ii-1)/(float)nocols)),
            (int) voffmxx - (int) origy - y +
               (int)((20.0/doubled)*cos(2.0*PI*(float)(ii-1)/(float)nocols)));
      }
      XSetForeground(display, drawgc, black_pixel);
      XSetLineAttributes(display, drawgc, 1, LineSolid, CapButt, JoinBevel);
      XDrawLine(display, XtWindow(drawwin), drawgc,
            x + (int) origx, (int) voffmxx - (int) origy - y,
            x + (int) origx, (int) (voffmxx - origy - (30.0/doubled)) - y);
      XDrawString(display, XtWindow(drawwin), drawgc,
            x + (int) origx, (int) (voffmxx - origy - (30.0/doubled)) - y,
             " 0", 2);
    }
  } else if (grayscale != 0) {
    /* we want grayscale for black and white */
    if (hsballoced) {
      XFreeColors(display, cmap, hsb, NUMXCOLS+1, 0L);
      hsballoced = 0;
    }
    if (hsb2alloced) {
      for (ii=0; ii<=NUMXHUES; ii++) {
        XFreeColors(display, cmap, hsb2[ii], NUMXBRIGHTS+1, 0L);
      }
      hsb2alloced = 0;
    }
    if (!grayalloced) {
      /* load up NUMXHUES colours of form R=G=B=i/NUMXHUES  */
GRAYS_AGAIN:
      for (ii=0; ii<=NUMXHUES; ii++) {
        coldef.red=coldef.green=coldef.blue=(short)(65535.0*(float)ii/(float)NUMXHUES);
        if (!XAllocColor(display, cmap, &coldef)) {
          if ((vclass == PseudoColor) && (cmap == DefaultColormap(display, screen_num))) {
            /* we have failed to allocate a color in the default colormap */
            /* since this is a pseudo-color visual, allocate our own colormap */
            /* instead */
            fprintf(stderr, "failed to alloc gray %d; creating private colormap\n", ii);
            XFreeColors(display, cmap, grays, ii, 0L);
            cmap = XCreateColormap(display, XtWindow(drawwin), default_visual, AllocNone);
            XAllocColor(display, cmap, &black);
            black_pixel = black.pixel;
            XAllocColor(display, cmap, &white);
            white_pixel = white.pixel;
            XtVaSetValues(drawwin, XtNcolormap, cmap, NULL);
            XtVaSetValues(pvshell, XtNcolormap, cmap, NULL);
            XtVaSetValues(viewwin, XtNcolormap, cmap, NULL);
            goto GRAYS_AGAIN;
          } else {
            fprintf(stderr, "failed to allocate gray %d\n", ii);
            exit(1);
          }
        }
        grays[ii] = coldef.pixel;
      }
      grayalloced = 1;
    }
  }

  /* do the norm scale at the right hand side (but only if */
  /* the fontsize is non-zero) */
  if (fontsize_loaded != 0.0) {
    if (!grayscale) {
      XDrawLine(display, XtWindow(drawwin), drawgc,
                (int)(origx + (pagewidth+12.0)/doubled),
                (int)(voffmxx - origy),
                (int)(origx + (pagewidth+12.0)/doubled),
                (int)(voffmxx - origy -scaling*maxnorm));
      XDrawLine(display, XtWindow(drawwin), drawgc,
                (int)(origx + (pagewidth+12.0)/doubled),
                (int)(voffmxx - origy),
                (int)(origx + (tick/doubled) + (pagewidth+12.0)/doubled),
                (int)(voffmxx - origy));
      sprintf(tickstring, "  %.2f", 0.0);
      XDrawString(display, XtWindow(drawwin), drawgc,
                 (int)(origx+ (tick/doubled) + (pagewidth+12.0)/doubled),
                 (int)(voffmxx - origy + fontsize_loaded/3.0),
                 tickstring, strlen(tickstring));
      XDrawLine(display, XtWindow(drawwin), drawgc,
                (int)(origx + (pagewidth+12.0)/doubled),
                (int)(voffmxx - origy -scaling*maxnorm),
                (int)(origx + (tick/doubled) + (pagewidth+12.0)/doubled),
                (int)(voffmxx - origy -scaling*maxnorm));
      sprintf(tickstring, "  %.2f", maxnorm);
      XDrawString(display, XtWindow(drawwin), drawgc,
                 (int)(origx+ (tick/doubled) + (pagewidth+12.0)/doubled),
                 (int)(voffmxx - origy -scaling*maxnorm+ fontsize_loaded/3.0),
                 tickstring, strlen(tickstring));
      XDrawLine(display, XtWindow(drawwin), drawgc,
                (int)(origx + (pagewidth+12.0)/doubled),
                (int)(voffmxx - origy -0.5*scaling*maxnorm),
                (int)(origx + (tick/doubled) + (pagewidth+12.0)/doubled),
                (int)(voffmxx - origy -0.5*scaling*maxnorm));
      sprintf(tickstring, "  %.2f", 0.5*maxnorm);
      XDrawString(display, XtWindow(drawwin), drawgc,
                 (int)(origx+ (tick/doubled) + (pagewidth+12.0)/doubled),
                 (int)(voffmxx - origy -0.5*scaling*maxnorm+ fontsize_loaded/3.0),
                 tickstring, strlen(tickstring));
    } else {
      /* do a grayscale on the right hand side */
      /* make it in blue though, because we dont have access to grays */
      dx = (tick/doubled);
      dy = (3.0*maxnorm);
      x = (int)((pagewidth+12.0)/doubled);
      XSetLineAttributes(display, drawgc, 5, LineSolid, CapButt, JoinBevel);
      for (ii=0; ii<50; ii++) {
        if (grayscale==1) {
          gray = (double)ii/50.0;
        } else {
          gray = (1.0 - (double)ii/50.0);
        }
        if (complexin) {
          XSetForeground(display, drawgc,
            hsb2[0][(int)(gray*(float)NUMXBRIGHTS)]);
        } else {
          XSetForeground(display, drawgc,
            grays[(int)(gray*(float)NUMXBRIGHTS)]);
        }

        XDrawLine(display, XtWindow(drawwin), drawgc,
                (int)(origx + x - dx),
                (int)(voffmxx - origy -((float)ii*((dy-5)/50.0)) +2.0),
                (int)(origx + x + dx),
                (int)(voffmxx - origy -((float)ii*((dy-5)/50.0)) +2.0));
      }
      XSetLineAttributes(display, drawgc, 1, LineSolid, CapButt, JoinBevel);
      XSetForeground(display, drawgc, black_pixel);
      XDrawRectangle(display, XtWindow(drawwin), drawgc,
                    (int)(origx + x - dx),
                    (int)(voffmxx - origy - dy)+6,
                    2*dx, (int) dy  - 1);
      sprintf(tickstring, "  %.2f", 0.0);
      XDrawString(display, XtWindow(drawwin), drawgc,
                 (int)(origx+ (tick/doubled) + (pagewidth+12.0)/doubled),
                 (int)(voffmxx - origy + fontsize_loaded/3.0),
                 tickstring, strlen(tickstring));
      sprintf(tickstring, "  %.2f", 1.0/(scaling*GRAYSCALE));
      XDrawString(display, XtWindow(drawwin), drawgc,
                 (int)(origx+ (tick/doubled) + (pagewidth+12.0)/doubled),
                 (int)(voffmxx - origy -3.0*maxnorm+ fontsize_loaded/3.0),
                 tickstring, strlen(tickstring));
      sprintf(tickstring, "  %.2f", 1.0/(scaling*2.0*GRAYSCALE));
      XDrawString(display, XtWindow(drawwin), drawgc,
                 (int)(origx+ (tick/doubled) + (pagewidth+12.0)/doubled),
                 (int)(voffmxx - origy -1.5*maxnorm+ fontsize_loaded/3.0),
                 tickstring, strlen(tickstring));

    }
  }
  if (reverse==-1) {
    origy += voffmax;
  }

}
void x_real_prolog(char *options, int norows, float voffmax, float vstep,
               float pagewidth, float *deltax, float scaling, float originx,
               float originy, float tick, float fontsize, float maxnorm,
               float doubled, int complexin, int grayscale, int reverse)
{
  x_complex_prolog(options, norows, voffmax, vstep, pagewidth,
                   deltax, scaling, originx, originy, tick, fontsize,
                   maxnorm, doubled, complexin, grayscale, reverse);
  XSetForeground(display, drawgc, black_pixel);
}
void x_eopage(float voffset, float pagewidth, float vstep,
               float doubled, float originx, float originy,
               float fontsize, int grayscale, int whichpage, int lr)
{
  if (grayscale)
    XSetForeground(display, drawgc, black_pixel);
  /* draw the vertical line */
  XDrawLine(display, XtWindow(drawwin), drawgc,
            (int) origx, (int) voffmxx - (int) origy,
            (int) origx, (int) (voffmxx - origy - voffset + vstep));
  /* draw the horizontal line */
  XDrawLine(display, XtWindow(drawwin), drawgc,
            (int) origx, (int) (voffmxx - origy),
            (int) (origx+pagewidth/doubled), (int) (voffmxx - origy));
  if (lr==1) {
    origx = originx / doubled;
  } else {
    origx = originx/doubled+(float)(lr-1)*(pagewidth+64+8.0*fontsize_loaded)/doubled;
  }

}
void x_epilog(int grayscale, float voffset, int whichpage,
               float pagewidth, float doubled)
{
  XSetForeground(display, drawgc, black_pixel);
  /* draw the vertical line */
  XDrawLine(display, XtWindow(drawwin), drawgc,
            (int) origx, (int) voffmxx - (int) origy,
            (int) origx, (int) (voffmxx - origy - voffset));
  /* draw the horizontal line */
  XDrawLine(display, XtWindow(drawwin), drawgc,
            (int) origx, (int) (voffmxx - origy),
            (int) (origx+pagewidth/doubled), (int) (voffmxx - origy));
  XFlush(display);
  XFreeGC(display, drawgc);
}

