.TH XREPLOT 1L "1 JAN 97" 

.SH NAME
xreplot \- a program for making evolving function graphs

.SH SYNOPSIS
.B xreplot
.RI [ options ]
.RI [ file ]

.SH GENERAL INFORMATION

.I xreplot
is a program for producing visualizations of evolving functions. It
features real and complex valued function support (using colour for
complex functions), and has both an function form and an intensity scale
output format. It includes options for multiple columns per page, 
reversed evolution direction and input sampling.

.I xreplot
reads input either from a file specified on the command line or
(if no such file is specified) from 
.I stdin
and produces either an X11 window or a standard PostScript file as output.

.SH USING XREPLOT

.I xreplot
has two input formats, and two main output formats. The input formats
reflect whether the input data is real or complex valued. For real
valued data, the input to
.I xreplot
is a series of lines, each line containing a (fixed) number of values.
Each line may be prefaced with a timestep value, enclosed in double 
quotes, and a line starting with a # character is ignored. For example:

.nf
  # comment line
  "0.1" 0.0 0.1 0.2 0.2 0.3 0.3 0.2 0.2 0.1 0.0
  "0.2" 0.0 0.1 0.2 0.3 0.4 0.4 0.3 0.2 0.1 0.0
  "0.3" 0.0 0.2 0.3 0.4 0.5 0.5 0.4 0.3 0.2 0.0
  "0.4" 0.0 0.3 0.2 0.1 0.3 0.2 0.1 0.2 0.3 0.0
  "0.5" 0.0 0.2 0.3 0.2 0.3 0.3 0.2 0.2 0.2 0.0
.fi

Each line must contain the same amount of data.

.I xreplot
may also receive complex-valued input. In this case, each line only
contains one complex value, in brackets. After a complete set of complex
values, the timestep value occurs on a line of its own. For example:

.nf
  (0.1, 0.2)
  (0.2, 0.3)
  (0.3, 0.2)
  (0.2, 0.3)
  (0.1, 0.2)
  0.1
  (0.2, 0.1)
  (0.3, 0.3)
  (0.3, 0.1)
  (0.3, 0.4)
  (0.2, 0.1)
  0.2
.fi

Each set of complex values must have the same number of values.

Once input, the real or complex-valued data may be output in two formats.
The first format produces a graph of the value of the function at each 
timestep, offset up the page as time progresses. 

The second format produces a grid of grayscaled rectangles across the
page for each timestep, with time progressing up the page.

In either case, complex valued input is depicted using colour. The
amplitude of a complex number gives its functional value, and the
argument gives its colour.

There are a large number of options which modify the behavior of
.I xreplot.
These options can be specified on the command line (by beginning
each option string with a '-' character).
For example, to change the scaling of the function at each timestep, 
you can use
.nf
	xreplot -n 5.4 data
.fi


.SH SUMMARY OF OPTIONS

The following command line options are available

.nf
-H            only send PostScript output to stdout
-a number     x coordinate for bottom left of plot, in points
-b number     y coordinate for bottom left of plot, in points
-y number     amount to vertically move between timesteps, in points
-t number     maximum y coordinate for each page, in points
-r number     maximum x coordinate for each line, in points
-n number     scaling factor for norm of function. For any particular 
              set of results, this will require modification to ensure 
              the resulting graph is clear
-f number     font size to use for labels, in points 
-m number     size of norm scale at side. This should be altered in 
              accordance with the -n option 
-p number     how often to print timestep labels (in terms of the 
              timestep values in the data)
-s integer    line sampling rate -- i.e. only plot every n-th line
              of data
-x number     minimum time between plots. Data with timestep values
              closer together than this will not be plotted. 
-k integer    gridpoint sampling rate. Only plot x values every n times.
-d integer    put two or more columns onto one page
-w integer    specify number of points per line for stdin
-g            produce gray scale output
-i            produce inverse gray scale output
-c            allow complex input (needs -w option)
-e integer    maximum number of pages to output
-q            produce output in reverse time direction
.fi

.SH  USING XREPLOT WITH LARGE FILES

.I xreplot
may be used with large data files. However, this will produce large 
PostScript output files. In these cases, it is recommended that the 
-s and -k options are used, so that only a sampling of the data is
actually printed. Finely sampled data in the x and t directions is 
usually unnecessary and indistiguishable from a more sparsely sampled
graph.

.SH  REGENERATING XREPLOT OUTPUT

Provided that the -H option is not set, 
.I xreplot
prints out the final option set when it exits.  In order to recreate
the same graph, re-use these options when starting 
.I xreplot.

If only PostScript output is required, use the -H option in 
combination with the given output set, redirecting standard 
output into a file.

.SH "SEE ALSO"
.PD 0
.PP
.I "xreplot - a program for making functional graphs",
.PP
.IR xyplot (1L),
.PD

.SH BUGS


Please report bugs and suggestions for improvements or further features
to David Drysdale at dmd@lurklurk.org


